#!/usr/bin/env node
'use strict';

var childProcess = require('child_process');
var inquirer = require('inquirer');
var chalk = require('chalk');
var path = require('path');
var fs = require('fs');
var spdxList = require('spdx-license-ids');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var childProcess__default = /*#__PURE__*/_interopDefaultLegacy(childProcess);
var inquirer__default = /*#__PURE__*/_interopDefaultLegacy(inquirer);
var chalk__default = /*#__PURE__*/_interopDefaultLegacy(chalk);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var spdxList__default = /*#__PURE__*/_interopDefaultLegacy(spdxList);

const repos = {
  defaultBoilerplate: 'https://github.com/backrunner/ts-cli-boilerplate.git'
};

/* eslint-disable no-console */
const licenseIds = spdxList__default['default'].map(item => item.toLowerCase());

const init = async () => {
  // input info
  console.log(chalk__default['default'].cyan('We need some necessary information to initialize your typescript cli project:'));
  const userInfo = await inquirer__default['default'].prompt([{
    type: 'input',
    name: 'name',
    message: 'Project Name: ',
    validate: v => {
      if (!v) {
        return 'Project name should not be empty.';
      }

      return true;
    }
  }, {
    type: 'input',
    name: 'cliName',
    message: 'Command line interface name: ',
    validate: v => {
      if (!v) {
        return 'Command line interface name should not be empty.';
      }

      return true;
    }
  }, {
    type: 'input',
    name: 'desc',
    message: 'Description: ',
    default: ''
  }, {
    type: 'input',
    name: 'version',
    message: 'Version: ',
    default: '0.0.0'
  }, {
    type: 'input',
    name: 'author',
    message: 'Author: ',
    default: ''
  }, {
    type: 'input',
    name: 'license',
    message: 'License: ',
    default: 'MIT',
    validate: v => {
      return licenseIds.includes(v.toLowerCase()) ? true : 'Invalid license.';
    }
  }, {
    type: 'confirm',
    name: 'useGit',
    message: 'Do you want to use Git to manage your project?',
    default: true
  }]);
  const gitOptions = await inquirer__default['default'].prompt([{
    type: 'input',
    name: 'commitMsg',
    message: 'First commit message: ',
    default: 'First commit'
  }]);
  const projectPath = path__default['default'].resolve(process.cwd(), `./${userInfo.name}`);

  if (fs__default['default'].existsSync(projectPath)) {
    const stat = fs__default['default'].statSync(projectPath);

    if (stat.isDirectory()) {
      const confirm = await inquirer__default['default'].prompt({
        type: 'confirm',
        name: 'v',
        message: `The folder named ${userInfo.name} already exists in the current directory, do you want to continue creating your project?`,
        default: false
      });

      if (!confirm.v) {
        return;
      }

      const delConfirm = await inquirer__default['default'].prompt({
        type: 'confirm',
        name: 'v',
        message: 'Do you want to empty the directory and then create the project?',
        default: false
      });

      if (delConfirm.v) {
        fs__default['default'].rmSync(projectPath, {
          recursive: true,
          force: true
        });
      }
    }
  } else {
    fs__default['default'].mkdirSync(projectPath);
  } // clone repo


  console.log(chalk__default['default'].cyan('Very well, next we will clone the boilerplate into the project folder.'));
  const branch = 'main';
  console.log(`git clone ${repos.defaultBoilerplate} -b ${branch} --depth 1 .`);
  childProcess__default['default'].execSync(`git clone ${repos.defaultBoilerplate} -b ${branch} --depth 1 .`, {
    stdio: 'inherit',
    cwd: projectPath
  }); // install dependencies

  console.log(chalk__default['default'].cyan('Looks good, we still need to do some final work to finish.'));
  childProcess__default['default'].execSync('npm install', {
    stdio: 'inherit',
    cwd: projectPath
  }); // modify package.json

  const packageInfoPath = path__default['default'].resolve(projectPath, './package.json');

  if (fs__default['default'].existsSync(packageInfoPath)) {
    const packageInfo = JSON.parse(fs__default['default'].readFileSync(packageInfoPath, {
      encoding: 'utf8'
    }));
    const {
      name,
      author,
      license,
      desc,
      version
    } = userInfo;
    Object.assign(packageInfo, {
      name,
      author,
      license,
      version,
      description: desc
    });

    if (!packageInfo.bin) {
      packageInfo.bin = {};
    }

    packageInfo.bin[userInfo.cliName] = './bin/cli.js';
    fs__default['default'].writeFileSync(packageInfoPath, JSON.stringify(packageInfo, null, '  '), {
      encoding: 'utf-8'
    });
  } else {
    console.log(chalk__default['default'].yellow('We did not find an available package.json, you may need to modify it manually by yourself.'));
  } // git


  if (userInfo.useGit) {
    console.log(chalk__default['default'].cyan('Creating git repository...'));
    childProcess__default['default'].execSync(`git init && git add . && git commit -m "${gitOptions.commitMsg}"`, {
      cwd: projectPath
    });
  } // done


  console.log(chalk__default['default'].green(`\n\n====================\n\nAll things done, you can build up your project now!\n\nHow to build your project: \n\ncd ./${userInfo.name}\nnpm run build\n\n`));
}; // execute


init();
